@@ -15,27 +15,26 @@ emLms <- function(model,
                   fix.estep = TRUE,
                   ...) {
   data <- model$data
-  model$data <- NULL # not needed in the model anymore
-  if (anyNA(data)) stop2("Remove or replace missing values from data")
+  stopif(anyNA(data), "Remove or replace missing values from data")
 
   # Initialization
-  logLikNew <- 0
-  logLikOld <- 0
+  logLikNew  <- 0
+  logLikOld  <- 0
   iterations <- 0     
-  thetaNew <- model$theta
+  thetaNew   <- model$theta
 
-  bestLogLik <- -Inf
-  bestP <- NULL 
-  bestTheta <- NULL
-  logLiks <- NULL
+  bestLogLik    <- -Inf
+  bestP         <- NULL 
+  bestTheta     <- NULL
+  logLiks       <- NULL
   logLikChanges <- NULL
 
-  run <- TRUE
+  run     <- TRUE
   doEstep <- TRUE
 
   while(run) {
     logLikOld <- logLikNew
-    thetaOld <- thetaNew
+    thetaOld  <- thetaNew
 
     if (doEstep) P <- estepLms(model = model, theta = thetaOld, data = data, ...)
   
@@ -43,21 +42,21 @@ emLms <- function(model,
                       max.step = max.step, epsilon = epsilon, 
                       optimizer = optimizer, control = control, ...)
 
-    logLikNew <- -mstep$objective
-    thetaNew <- unlist(mstep$par)
-    iterations <- iterations + 1
-    logLiks <- c(logLiks, logLikNew)
+    logLikNew     <- -mstep$objective
+    thetaNew      <- unlist(mstep$par)
+    iterations    <- iterations + 1
+    logLiks       <- c(logLiks, logLikNew)
     logLikChanges <- c(logLikChanges, logLikNew - logLikOld)
 
     if (verbose) {
       cat(sprintf("EM: Iteration = %5d, LogLik = %11.2f, Change = %10.3f\n",
-            iterations, logLikNew, logLikNew - logLikOld))
+                  iterations, logLikNew, logLikNew - logLikOld))
     }
 
     if (logLikNew > bestLogLik) {
       bestLogLik <- logLikNew
-      bestP <- P 
-      bestTheta <- thetaOld
+      bestP      <- P 
+      bestTheta  <- thetaOld
     }
 
     if (abs(logLikOld - logLikNew) < convergence) run <- FALSE
@@ -69,9 +68,10 @@ emLms <- function(model,
 
     if (doEstep && fix.estep && runningAverage(logLikChanges, n = 30) < 0 && 
         nNegativeLast(logLikChanges, n = 30) >= 15 && iterations > 200) {
-      doEstep <- FALSE  
-      P <- bestP 
+      doEstep  <- FALSE  
+      P        <- bestP 
       thetaNew <- bestTheta
+
       warning2("EM algorithm is not converging. ", 
                "Attempting to fix prior probabilities from E-step\n", 
                "you might want to increase the convergence (i.e., less strict) criterion (see 'help(modsem_da)')")
@@ -84,13 +84,14 @@ emLms <- function(model,
                     verbose = verbose, control = control, ...)
     
   coefficients <- final$par
-  finalModel <- fillModel(model, coefficients, fillPhi = TRUE,
-                          method = "lms")
+  lavCoefs     <- getLavCoefs(model = model, theta = coefficients, method = "lms")
+  finalModel      <- fillModel(model, coefficients, fillPhi = TRUE, method = "lms")
+  info            <- model$info
 
-  emptyModel <-  getEmptyModel(parTable = model$parTable, 
-                               cov.syntax = model$cov.syntax,
-                               parTableCovModel = model$covModel$parTable,
-                               method = "lms")
+  emptyModel <- getEmptyModel(parTable = model$parTable, 
+                              cov.syntax = model$cov.syntax,
+                              parTableCovModel = model$covModel$parTable,
+                              method = "lms")
 
   finalModel$matricesNA <- emptyModel$matrices
   finalModel$covModelNA <- emptyModel$covModel
@@ -103,40 +104,46 @@ emLms <- function(model,
                     EFIM.parametric = EFIM.parametric, verbose = verbose,
                     FIM = FIM, robust.se = robust.se, epsilon = epsilon,
                     NA__ = -999)
-  SE <- calcSE_da(calc.se = calc.se, vcov = FIM$vcov, theta = coefficients, NA__ = -999)
+  SE <- calcSE_da(calc.se = calc.se, FIM$vcov, rawLabels = FIM$raw.labels, 
+                  NA__ = -999)
+  modelSE <- getSE_Model(model, se = SE, method = "lms",
+                         n.additions = FIM$n.additions)
 
-  modelSE <- fillModel(replaceNonNaModelMatrices(model, value = -999), 
-                       theta = SE, method = "lms")
   finalModel$matricesSE <- modelSE$matrices
   finalModel$covModelSE <- modelSE$covModel
 
-  parTable <- modelToParTable(finalModel, method = "lms")
+  parTable <- modelToParTable(finalModel, coefs = lavCoefs,
+                              se = SE, method = "lms")
 
-  parTable$z.value <- parTable$est / parTable$std.error
-  parTable$p.value <- 2 * stats::pnorm(-abs(parTable$z.value))
+  parTable$z.value  <- parTable$est / parTable$std.error
+  parTable$p.value  <- 2 * stats::pnorm(-abs(parTable$z.value))
   parTable$ci.lower <- parTable$est - 1.96 * parTable$std.error
   parTable$ci.upper <- parTable$est + 1.96 * parTable$std.error
 
   # convergence of em
   if (iterations == max.iter) convergence <- FALSE else convergence <- TRUE
 
-  out <- list(model = finalModel, 
-              method = "lms",
+  out <- list(model     = finalModel, 
+              method    = "lms",
               optimizer = paste0("EM-", optimizer),
-              data = data,
-              theta = coefficients,
-              parTable = parTable,
+              data      = data,
+              theta     = coefficients,
+              coefs     = lavCoefs,
+              parTable  = parTable,
+
               originalParTable = model$parTable,
-              logLik = -final$objective, 
-              iterations = iterations,
+
+              logLik      = -final$objective, 
+              iterations  = iterations,
               convergence = convergence, 
-              type.se = typeSE,
-              info.quad = getInfoQuad(model$quad),
+              type.se     = typeSE,
+              info.quad   = getInfoQuad(model$quad),
+
               type.estimates = "unstandardized",
-              FIM = FIM$FIM,
-              vcov = FIM$vcov,
+
+              FIM         = FIM$FIM,
+              vcov        = FIM$vcov,
               information = FIM$type)
 
-  class(out) <- "modsem_lms"
   out
 }
