@@ -15,16 +15,17 @@ estQml <- function(model,
   final <- mstepQml(model = model, theta = startTheta, max.iter = max.iter, 
                     convergence = convergence, epsilon = epsilon,
                     verbose = verbose, optimizer = optimizer, ...)
-  coefficients <- final$par
-  finalModel <- fillModel(model, coefficients)
 
-  info <- model$info
+  coefficients <- final$par
+  lavCoefs     <- getLavCoefs(model = model, theta = coefficients, method = "lms")
+  finalModel      <- fillModel(model, coefficients)
+  info            <- model$info
 
   finalModel <- fillModel(model, coefficients, method = "qml")
-  emptyModel <-  getEmptyModel(parTable = model$parTable, 
-                               cov.syntax = model$cov.syntax,
-                               parTableCovModel = model$covModel$parTable,
-                               method = "qml")
+  emptyModel <- getEmptyModel(parTable = model$parTable, 
+                              cov.syntax = model$cov.syntax,
+                              parTableCovModel = model$covModel$parTable,
+                              method = "qml")
   finalModel$matricesNA <- emptyModel$matrices
   finalModel$covModelNA <- emptyModel$covModel
 
@@ -36,45 +37,47 @@ estQml <- function(model,
                     EFIM.parametric = EFIM.parametric, verbose = verbose,
                     FIM = FIM, robust.se = robust.se, NA__ = -999, 
                     epsilon = epsilon)
-  SE <- calcSE_da(calc.se = calc.se, FIM$vcov, theta = coefficients, NA__ = -999)
+  SE <- calcSE_da(calc.se = calc.se, FIM$vcov, rawLabels = FIM$raw.labels, 
+                  NA__ = -999)
+  modelSE <- getSE_Model(model, se = SE, method = "qml",
+                         n.additions = FIM$n.additions)
 
-  modelSE <- fillModel(replaceNonNaModelMatrices(model, value = -999), 
-                       theta = SE, method = "lms")
-
-  modelSE <- fillModel(replaceNonNaModelMatrices(model, value = -999), 
-                       SE, method = "qml")
   finalModel$matricesSE <- modelSE$matrices
   finalModel$covModelSE <- modelSE$covModel
 
-  parTable <- modelToParTable(finalModel, method = "qml")
+  parTable <- modelToParTable(finalModel, coefs = lavCoefs,
+                              se = SE, method = "qml")
 
-  parTable$z.value <- parTable$est / parTable$std.error
-  parTable$p.value <- 2 * stats::pnorm(-abs(parTable$z.value))
+  parTable$z.value  <- parTable$est / parTable$std.error
+  parTable$p.value  <- 2 * stats::pnorm(-abs(parTable$z.value))
   parTable$ci.lower <- parTable$est - 1.96 * parTable$std.error
   parTable$ci.upper <- parTable$est + 1.96 * parTable$std.error
   
-  if (final$iterations >= max.iter) {
-      warning2("Maximum number of iterations was reached, ",
-               "model estimation might not have converged.")
-  }
+  warnif(final$iterations >= max.iter,
+         "Maximum number of iterations was reached, ",
+         "model estimation might not have converged.")
 
-  out <- list(model = finalModel, 
-              method = "qml",
+  out <- list(model     = finalModel,
+              method    = "qml",
               optimizer = optimizer,
-              data  = model$data,
-              theta = coefficients,
-              parTable = parTable,
+              data      = model$data,
+              theta     = coefficients,
+              coefs     = lavCoefs,
+              parTable  = parTable,
+
               originalParTable = model$parTable,
-              logLik = -final$objective, 
-              iterations = final$iterations,
+
+              logLik      = -final$objective, 
+              iterations  = final$iterations,
               convergence = final$convergence, 
-              type.se = typeSE,
+              type.se     = typeSE,
+
               type.estimates = "unstandardized",
-              info.quad = NULL,
-              FIM = FIM$FIM,
-              vcov = FIM$vcov,
+
+              info.quad   = NULL,
+              FIM         = FIM$FIM,
+              vcov        = FIM$vcov,
               information = FIM$type)
 
-  class(out) <- "modsem_qml"
   out
 }
