@@ -1,31 +1,17 @@
-
-expressionType <- function(op, lhs, rhs) {
-if (is.null(op) & is.null(rhs)) {
-    if (is.null(lhs)) {
-      stop2("Missing lhs")
-    }
-
-  }
-}
-
-
-
 createExprNode <- function(node, lhs = NULL, rhs = NULL,
                            priority = 1) {
   structure(list(node = node,
-                 lhs = lhs,
-                 rhs = rhs),
-            class = "exprNode",
+                 lhs  = lhs,
+                 rhs  = rhs),
+
+            class    = "exprNode",
             priority = priority)
 }
 
 
-
 getMinTokenPriority <- function(listTokens, min = NA) {
   if (is.null(listTokens) || length(listTokens) == 0) {
-    if (is.na(min)) {
-      stop2("Unable to find minimum priority for tokens")
-    }
+    stopif(is.na(min), "Unable to find minimum priority for tokens")
     return(min)
   }
 
@@ -37,16 +23,12 @@ getMinTokenPriority <- function(listTokens, min = NA) {
 }
 
 
-
 chooseToken <- function(listTokens, i = 1, chosenTokenIdx = NULL,
                         leftClosures = list()) {
 
   if (is.null(listTokens) || i > length(listTokens)) {
-    if (is.null(chosenTokenIdx)) {
-      if (length(leftClosures) > 0) {
-        stop2("Unmatched left bracket", last(leftClosures))
-      }
-    }
+    stopif(is.null(chosenTokenIdx) && length(leftClosures) > 0,
+           "Unmatched left bracket", last(leftClosures))
     return(chosenTokenIdx)
   }
   token <- listTokens[[i]]
@@ -55,9 +37,9 @@ chooseToken <- function(listTokens, i = 1, chosenTokenIdx = NULL,
     leftClosures <- appendToList(leftClosures, token)
 
   } else if (is.RightClosure(token)) {
-    if (length(leftClosures) == 0) {
-      stop2("Unmatched right bracket", highlightErrorToken(token))
-    }
+    stopif(length(leftClosures) == 0, "Unmatched right bracket", 
+           highlightErrorToken(token))
+
     leftClosures <- leftClosures[-1]
     if (length(leftClosures) == 0) {
       return(i)
