@@ -26,18 +26,15 @@ plot_interaction("X", "Z", "Y", "X:Z", -3:3, c(-0.2, 0), est1)
 tpb <- "
 # Outer Model (Based on Hagger et al., 2007)
   ATT =~ att1 + att2 + att3 + att4 + att5
-  LSN =~ sn1 + sn2
+  SN =~ sn1 + sn2
   PBC =~ pbc1 + pbc2 + pbc3
   INT =~ int1 + int2 + int3
   BEH =~ b1 + b2
 
 # Inner Model (Based on Steinmetz et al., 2011)
-  # Causal Relationsships
-  INT ~ ATT + LSN + PBC
+  INT ~ ATT + SN + PBC
   BEH ~ INT + PBC
-  # BEH ~ ATT:PBC
   BEH ~ PBC:INT
-  # BEH ~ PBC:PBC
 "
 
 est2 <- modsem(tpb, TPB, method = "lms")
