@@ -1,20 +1,20 @@
 devtools::load_all()
 tpb <- ' 
 # Outer Model (Based on Hagger et al., 2007)
-  LATT =~ att1 + att2 + att3 + att4 + att5
-  LSN =~ sn1 + sn2
-  LPBC =~ pbc1 + pbc2 + pbc3
-  LINT =~ int1 + int2 + int3
-  LBEH =~ b1 + b2
+  ATT =~ att1 + att2 + att3 + att4 + att5
+  SN =~ sn1 + sn2
+  PBC =~ pbc1 + pbc2 + pbc3
+  INT =~ int1 + int2 + int3
+  BEH =~ b1 + b2
 
 # Inner Model (Based on Steinmetz et al., 2011)
   # Covariances
-  LATT ~~ cAsn * LSN + cApbc * LPBC
-  LPBC ~~ cPbcSn * LSN 
+  ATT ~~ SN + PBC
+  PBC ~~ SN 
   # Causal Relationsships
-  LINT ~ gIa * LATT + gIsn * LSN + gIpbc * LPBC
-  LBEH ~ LINT + LPBC 
-  LBEH ~ LINT:LPBC  
+  INT ~ a * ATT + b * SN + c * PBC
+  BEH ~ INT + PBC 
+  BEH ~ INT:PBC  
 '
 
 method <- c("ca", "rca", "uca", "dblcent")
